/**
 * 历史组件
 */
import React, { Component } from 'react';
import { Motion, spring } from 'react-motion';
import {getData} from '@icode-run/component-layout/utils/tools';
import styles from './index.module.less';

export default class History extends Component {
  static defaultProps = {
    data: [],
    pending: false,
  }
  state = {
    current: 0,
    offset: 0,
  }
  nodes = []
  componentDidMount() {
    this.nodeOffsets = this.nodes.map(d => d.getBoundingClientRect().height);
    this.updateOffset();
  }
  componentWillReceiveProps(nextProps) {
    if (this.props.pending && !nextProps.pending) {
      this.nodeOffsets = this.nodes.map(d => d.getBoundingClientRect().height);
    }
  }
  updateOffset() {
    const { current } = this.state;
    const newCurrent = current >= this.nodes.length ? 0 : current + 1;
    this.setState({
      current: newCurrent,
      offset: newCurrent === 0 ? 0 : this.nodeOffsets.reduce((lastValue, v, index) => {
        if (current >= index) {
          return lastValue + v;
        }
        return lastValue;
      }, 0),
    });
  }
  reorderData = () => {
    setTimeout(() => {
      this.updateOffset();
    }, 2000);
  }
  render() {
    const { className = '', data, dataKeys } = this.props;
    let values = getData(data, dataKeys);
    
    const props = {
      className: `${styles.componentHistory} ${className}`,
    };
    const { offset } = this.state;
    return (
      <div {...props}>
        <Motion
          style={{
            y: offset === 0 ? 0 : spring(offset)
          }}
          onRest={this.reorderData}
        >
        {
          interpolatingStyle => (
            <ul className="history-content" style={{
              transform: `translateY(-${interpolatingStyle.y}px)`,
            }}>
              {
                values.map((d, i) => (
                  <li
                    className={!d[0] ? 'no-date' : ''}
                    key={d[0] || i}
                    ref={node => this.nodes[i] = node}
                  >
                    <em className="date">{d[0]}</em>
                    <p className="description">{d[1]}</p>
                  </li>
                ))
              }
              {
                values.map((d, i) => (
                  <li
                    className={!d[0] ? 'no-date' : ''}
                    key={d[0] || i}
                  >
                    <em className="date">{d[0]}</em>
                    <p className="description">{d[1]}</p>
                  </li>
                ))
              }
            </ul>
          )
        }
        </Motion>
      </div>
    );
  }
}